<?php
namespace OPAY\PaymentGateway\Controller\Callback;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use OPAY\PaymentGateway\Model\Ui\ConfigProvider;
use OPAY\PaymentGateway\Helper\RequestHandler;

/* Solves magento 2-3-0 backwards compatibility issue (starting in 2-3-0 POST actions bus implement CsrfAwareActionInterface) */
if (interface_exists("\Magento\Framework\App\CsrfAwareActionInterface")) {
    class Callback extends Action implements CsrfAwareActionInterface
    {
        /**
         * @var \OPAY\PaymentGateway\Helper\RequestHandler
         */
        private $requestHandler;

        /**
         * Constructor
         *
         * @param Context $context
         * @param RequestHandler $requestHandler
         */
        public function __construct(Context $context, RequestHandler $requestHandler)
        {
            $this->requestHandler = $requestHandler;
            parent::__construct($context);
        }

        /**
         * {@inheritdoc}
         *
         * @see \Magento\Framework\App\ActionInterface
         */
        public function execute()
        {
            return $this->requestHandler->handleCallback();
        }

        /**
         * @inheritDoc
         */
        public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException {
            return null;
        }

        /**
         * @inheritDoc
         */
        public function validateForCsrf(RequestInterface $request): ?bool
        {
            return true;
        }
    }
}

else {
    class Callback extends Action
    {
        /**
         * @var \OPAY\PaymentGateway\Helper\RequestHandler
         */
        private $requestHandler;

        /**
         * Constructor
         *
         * @param Context $context
         * @param RequestHandler $requestHandler
         */
        public function __construct(Context $context, RequestHandler $requestHandler)
        {
            $this->requestHandler = $requestHandler;
            parent::__construct($context);
        }

        /**
         * {@inheritdoc}
         *
         * @see \Magento\Framework\App\ActionInterface
         */
        public function execute()
        {
            return $this->requestHandler->handleCallback();
        }
    }
}