<?php
namespace OPAY\PaymentGateway\Controller\Redirect;

use Exception;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Мagento\Framework\App\ObjectManager;
use OPAY\PaymentGateway\Model\Ui\ConfigProvider;

class Redirect extends Action
{
    const APP_NAME = 'Magento2';
    const WIDGET_VERSION = '2.1.2';

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $session;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    private $url;

    /**
     * @var \Magento\Framework\Locale\ResolverInterface
     */
    private $localeResolver;

    /**
     * @var \OpayGateway
     */
    private $opayGateway;
    private $om;

    /**
     * Contructor
     */
    public function __construct(Context $context)
    {
        $this->om = $context->getObjectManager();

        $this->session        = $this->om->get('Magento\Checkout\Model\Session');
        $this->url            = $this->om->get('Magento\Framework\UrlInterface');
        $this->localeResolver = $this->om->get('Magento\Framework\Locale\ResolverInterface');
        $this->opayGateway    = $this->om->get('OPAY\PaymentGateway\Helper\OpayGatewayFactory')->create();

        parent::__construct($context);
    }

    /**
     * {@inheritdoc}
     *
     * @see \Magento\Framework\App\ActionInterface
     */
    public function execute()
    {
        $order = $this->session->getLastRealOrder();
        $payment = $order->getPayment();
        $method = $payment->getMethodInstance();

        if ($method->getCode() != ConfigProvider::CODE) {
            return $this->_redirect($this->_url->getBaseUrl());
        }

        $additionalInfo = $payment->getAdditionalInformation();
        $billingAddress = $order->getBillingAddress();
        $locale = $this->localeResolver->getLocale();
        $language = strtoupper(substr($locale, 0, 2));
        $total = $order->getGrandTotal();

        $params = array(
            'website_id'      => $method->getConfigData('website_id'),
            'order_nr'        => $order->getRealOrderId(),
            'redirect_url'    => $this->url->getUrl('opay/callback/userredirect'),
            'web_service_url' => $this->url->getUrl('opay/callback/callback'),
            'standard'        => 'opay_8.1',
            'country'         => $billingAddress->getCountryId(),
            'language'        => $language,
            'amount'          => intval(number_format($total * 100, 0, '', '')),
            'currency'        => $order->getOrderCurrencyCode(),
            'c_email'         => $order->getCustomerEmail(),
            'c_mobile_nr'     => $billingAddress->getTelephone()
        );

        if ($method->getConfigData('test_mode')) {
            $params['test'] = $method->getConfigData('user_id');
        }

        if (isset($additionalInfo['channel_name'])) {
            $params['pass_through_channel_name'] = $additionalInfo['channel_name'];
        }

        $this->appendMetadataToParams($params);
        $params = $this->opayGateway->signArrayOfParameters($params);

        try {
            $url = sprintf(
                'https://gateway.opay.lt/pay/?encoded=%s',
                $this->opayGateway->convertArrayOfParametersToEncodedString($params)
            );
        } catch (\Exception $exc) {
            // TODO log exception
            return $this->_redirect($this->_url->getBaseUrl());
        }

        // Adding a comment to history
        if (isset($additionalInfo['channel_name'])) {
            $comment = sprintf(
                __('Customer chose payment method: %s.'),
                $additionalInfo['channel_name']
            );
        } else {
            $comment = __('Customer redirected to external OPAY page.');
        }
        try {
            $order->addStatusHistoryComment($comment)->save();
        } catch (\Exception $exc) { }

        header(sprintf('Location: %s', $url));
    }

    private function appendMetadataToParams(&$params)
    {
        if (array_key_exists('metadata', $params)) {
            unset($params['metadata']);
        }

        try {
            $productMetadata = $this->om->get('Magento\Framework\App\ProductMetadataInterface');
            $params['metadata']['app_version'] =  $productMetadata->getVersion();
        } catch (Exception $exception) {
            // Do nothing to avoid any risks to break payment process
        }

        $params['metadata']['app_name'] = self::APP_NAME;
        $params['metadata']['widget_version'] = self::WIDGET_VERSION;
    }
}

