<?php
namespace OPAY\PaymentGateway\Gateway\Request;

use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Framework\HTTP\PhpEnvironment\Request;
use OPAY\PaymentGateway\Helper\OpayGatewayFactory;

class DataRequest implements BuilderInterface
{
    /**
     * @var \OpayGateway
     */
    private $opayGateway;

    /**
     * Contructor
     */
    public function __construct(OpayGatewayFactory $opayGatewayFactory)
    {
        $this->opayGateway = $opayGatewayFactory->create();
    }

    /**
     * {@inheritdoc}
     *
     * @see \Magento\Payment\Gateway\Request\BuilderInterface
     */
    public function build(array $buildSubject)
    {
        if (!isset($buildSubject['request'])
            || !$buildSubject['request'] instanceof Request
        ) {
            throw new \InvalidArgumentException('Request object should be provided');
        }

        $request = $buildSubject['request'];
        $opay = $this->opayGateway;

        if ($request->getPostValue('encoded')) {
            $params = $opay->convertEncodedStringToArrayOfParameters(
                $request->getPostValue('encoded')
            );
        } elseif ($request->getQueryValue('encoded')) {
            $params = $opay->convertEncodedStringToArrayOfParameters(
                $request->getQueryValue('encoded')
            );
        } elseif ($request->getPostValue('password_signature')
               || $request->getPostValue('rsa_signature')
        ) {
            $params = $request->getPostValue();
        } elseif ($request->getQueryValue('password_signature')
               || $request->getQueryValue('rsa_signature')
        ) {
            $params = $request->getQueryValue();
        }

        if (!isset($params) || !$opay->verifySignature($params)) {
            throw new \RuntimeException('Request is not valid gateway request');
        }

        return $params;
    }
}
