/*browser:true*/
/*global define*/
define(
    [
        'jquery',
        'Magento_Checkout/js/view/payment/default',
        'mage/url',
        'mage/translate'
    ],
    function ($, Component, url, $t) {
        'use strict';

        return Component.extend({
            defaults: {
                template: 'OPAY_PaymentGateway/payment/form',
                redirectAfterPlaceOrder: false,
                redirectUrl: "opay/redirect/redirect/"
            },

            getTitle: function () {
                var config = window.checkoutConfig.payment.opay_gateway;
                if (config.payment_gateway_name) {
                    return config.payment_gateway_name
                } else {
                    return $t("OPAY Payment Gateway");
                }
            },

            getIconHeight: function() {
                return window.checkoutConfig.payment.opay_gateway.icon_height;
            },

            getChannels: function() {
                return window.checkoutConfig.payment.opay_gateway.channels;
            },

            showChannelsGrouped: function() {
                var config = window.checkoutConfig.payment.opay_gateway;
                return config.channels && config.show_payment_methods == "in_checkout_grouped";
            },

            showChannels: function() {
                var config = window.checkoutConfig.payment.opay_gateway;
                return config.channels && config.show_payment_methods == "in_checkout";
            },

            validate: function() {
                if (this.showChannels() || this.showChannelsGrouped()) {
                    var channelName = this.getSellectedChannelName();
                    if (channelName == undefined || !channelName) {
                        this.messageContainer.addErrorMessage({
                            "message": $t("Please select payment method.")
                        });
                        return false;
                    }
                }
                return true;
            },

            getSellectedChannelName: function() {
                var channelSelector = 'input[name=payment\\[method_channel\\]]:checked'
                  , formSelector = '#payment-form-' + this.getCode();
                return $(channelSelector, formSelector).val();
            },

            afterPlaceOrder: function() {
                window.location.replace(url.build(this.redirectUrl));
            },

            getData: function() {

                var additionalData = {};

                if (this.showChannels() || this.showChannelsGrouped()) {
                    additionalData.channel_name = this.getSellectedChannelName();
                }

                return {
                    "method": this.item.method,
                    "additional_data": additionalData
                };
            }
        });
    }
);
